/**************************************************
 *
 *
 **************************************************
 * This code was created by Peter Harvey @ CodeByDesign.
 * Released under GPL 18.FEB.99
 *
 * Contributions from...
 * -----------------------------------------------
 * Peter Harvey		- pharvey@codebydesign.com
 **************************************************/
#include "classISQL.h"

#include "run.xpm"
#include "new.xpm"
#include "open.xpm"
#include "save.xpm"

classISQL::classISQL( SQLHDBC hDbc, QString qsDataSource, QWidget *parent,
const char *name )
	: QWidget( parent, name )
{
    QString         qsCaption;
	QFont 			qf( "Fixed", 12 );

	qf.setFixedPitch( true );

    this->hDbc					= hDbc;
    this->qsDataSource  		= qsDataSource.stripWhiteSpace();
	qsSQLFileName				= "";
	qsResultsFileName			= "";

	qsCaption.sprintf( "ISQL %s", (const char *)qsDataSource );

    setCaption( qsCaption );

	mbMain = new QMenuBar( this, "mbMain" );
	mbMain->setGeometry( 0, 0, 590, 30 );
	mbMain->setMinimumSize( 0, 0 );
	mbMain->setMaximumSize( 32767, 32767 );
	mbMain->setFocusPolicy( QWidget::NoFocus );
	mbMain->setBackgroundMode( QWidget::PaletteBackground );
	mbMain->setFontPropagation( QWidget::NoChildren );
	mbMain->setPalettePropagation( QWidget::NoChildren );
	mbMain->setFrameStyle( 34 );
	mbMain->setLineWidth( 2 );

	txtSQL = new QMultiLineEdit( this, "txtSQL" );
	txtSQL->setFocusPolicy( QWidget::StrongFocus );
	txtSQL->setBackgroundMode( QWidget::PaletteBase );
	txtSQL->setFontPropagation( QWidget::SameFont );
	txtSQL->setPalettePropagation( QWidget::SameFont );
	txtSQL->insertLine( "" );
	txtSQL->setReadOnly( FALSE );
	txtSQL->setOverwriteMode( FALSE );
	txtSQL->setFont( qf );


	pTabBar		= new QTabBar( this );
	pTabBar->setGeometry( 0, 30, 150, 20 );

	QTab *pTab	= new QTab();
	pTab->label = "SQL";
	pTabBar->addTab( pTab );

	pTab 		= new QTab();
	pTab->label = "Results";
	pTabBar->addTab( pTab );

	connect( pTabBar, SIGNAL(selected(int)), SLOT(ChangeTextType(int)) );

	cmdExecSQL = new QPushButton( this, "cmdExecSQL" );
	cmdExecSQL->setGeometry( 160, 30, 35, 20 );
	cmdExecSQL->setMinimumSize( 0, 0 );
	cmdExecSQL->setMaximumSize( 32767, 32767 );
	cmdExecSQL->setPixmap( QPixmap( run_xpm ) );

	connect( cmdExecSQL, SIGNAL(clicked()), SLOT(ExecSQL()) );
	cmdExecSQL->setFocusPolicy( QWidget::TabFocus );
	cmdExecSQL->setBackgroundMode( QWidget::PaletteBackground );
	cmdExecSQL->setFontPropagation( QWidget::NoChildren );
	cmdExecSQL->setPalettePropagation( QWidget::NoChildren );
	cmdExecSQL->setAutoRepeat( FALSE );
	cmdExecSQL->setAutoResize( FALSE );

	txtResults = new QMultiLineEdit( this, "txtResults" );
	txtResults->setFocusPolicy( QWidget::StrongFocus );
	txtResults->setBackgroundMode( QWidget::PaletteBase );
	txtResults->setFontPropagation( QWidget::SameFont );
	txtResults->setPalettePropagation( QWidget::SameFont );
	txtResults->insertLine( "" );
	txtResults->setReadOnly( FALSE );
	txtResults->setOverwriteMode( FALSE );
	txtResults->setFont( qf );

	resize( 590,400 );
	setMinimumSize( 50, 50 );
	setMaximumSize( 32767, 32767 );

    // SETUP MAIN MENU
    QPopupMenu *pFile = new QPopupMenu();
    pFile->insertItem( QPixmap( new_xpm ), "&New",  this, SLOT(NewSQL()),
        CTRL+Key_N );
    pFile->insertItem( QPixmap( open_xpm ), "&Open",  this,
        SLOT(OpenSQL()), CTRL+Key_O );
    pFile->insertItem( QPixmap( save_xpm ), "&Save", this, SLOT(SaveSQL()),
        CTRL+Key_S );
    pFile->insertItem( "Save &As", this, SLOT(SaveAsSQL()), CTRL+Key_A );

    mbMain->insertItem( "&File", pFile );
    mbMain->setSeparator( QMenuBar::InWindowsStyle );
    txtResults->hide();

	connect( parent, SIGNAL(changedSize(int,int)), SLOT(resize(int,int)) );
	
	lblFileName	= new QLabel( this );
	lblFileName->setFrameStyle( QFrame::Panel | QFrame::Sunken );

	resize( parent->width(), parent->height() );
}


classISQL::~classISQL()
{
}

void classISQL::ExecSQL()
{
    SQLHSTMT    hStmt;
    QString     qsHorizSep;
    SQLINTEGER  nRowsAffected;
    SWORD       nColumns;
    int         iRC                 = 0;
/* TIM - return error messages if provided */
	char	szState[10];
	char	szBuf[700];
	char	szErrString[800];
	SQLRETURN retcode;
		
	// CREATE A STATEMENT
    iRC = SQLAllocStmt( hDbc, &hStmt );
    if( SQL_SUCCESS != iRC )
		QMessageBox::critical( (QWidget *)this, "Data Manager", "Failed: SQLAllocStmt " );

    if( SQL_SUCCESS != (iRC=SQLPrepare(hStmt,
        (SQLCHAR*)(txtSQL->text().data()), SQL_NTS)) )
    {
    	retcode = SQLError(SQL_NULL_HENV, hDbc, hStmt, (SQLCHAR*)szState,
            NULL, (SQLCHAR*)szBuf, 700, NULL);
	if (retcode == SQL_SUCCESS)
		QMessageBox::critical( (QWidget *)this, "Data Manager", szBuf);
	else
		QMessageBox::critical( (QWidget *)this, "Data Manager", "Failed: SQLPrepare " );
    }

    // EXECUTE
    if( SQL_SUCCESS != (iRC=SQLExecute(hStmt)) )
    {
    	retcode = SQLError(SQL_NULL_HENV, hDbc, hStmt, (SQLCHAR*)szState,
            NULL, (SQLCHAR*)szBuf, 700, NULL);
	if (retcode == SQL_SUCCESS)
		QMessageBox::critical( (QWidget *)this, "Data Manager", szBuf);
	else
		QMessageBox::critical( (QWidget *)this, "Data Manager", "Failed: SQLExecute " );
    }

    // GET NUMBER OF ROWS AFFECTED
    if ( (iRC = SQLRowCount( hStmt, &nRowsAffected )) != SQL_SUCCESS )
        nRowsAffected = -1;

    // GET NUMBER OF COLUMNS RETURNED
    if ( (iRC = SQLNumResultCols( hStmt, &nColumns )) != SQL_SUCCESS )
        nColumns = -1;

    txtResults->setAutoUpdate( FALSE );
    txtResults->clear();

    // GET A RESULTS HEADER (column headers)
    getResultsHeader( hStmt, nColumns, &qsHorizSep );
    // GET A RESULTS BODY (data)
    getResultsBody( hStmt, nColumns );

    // FREE STATEMENT
    iRC = SQLFreeStmt( hStmt, SQL_DROP );
    if( SQL_SUCCESS != iRC )
		QMessageBox::critical( (QWidget *)this, "Data Manager", "Failed: SQLFreeStmt " );

    pTabBar->setCurrentTab( 1 );
    txtResults->show();
    txtSQL->hide();

    txtResults->append( qsHorizSep );
    qsHorizSep.sprintf( "\n\n%d rows affected", nRowsAffected );
    txtResults->append( qsHorizSep );

    txtResults->setAutoUpdate( TRUE );
    txtResults->repaint();

}

int classISQL::getResultsHeader( SQLHSTMT hStmt, SWORD nColumns, QString
*pqsHorizSep )
{
    QString 		qsColumnHeader		= "";
	QString			qsColumn			= "";
	QString			qsColumnName		= "";
	int				nCol;
	unsigned int	nMaxLength			= 10;
	SQLCHAR			szColumnName[101]	= "";	

    *pqsHorizSep = "";
	for( nCol = 1; nCol <= nColumns; nCol++ )
	{
		SQLColAttribute( hStmt, nCol, SQL_DESC_DISPLAY_SIZE, 0, 0, 0, &nMaxLength );
		SQLColAttribute( hStmt, nCol, SQL_DESC_LABEL, szColumnName,
sizeof(szColumnName), 0, 0 );
		qsColumnName = (const char *)szColumnName;
		qsColumn.fill( '-', max( nMaxLength, qsColumnName.length() ) );
		qsColumn += "-";
		*pqsHorizSep += ( "+" + qsColumn );
		qsColumn.sprintf( "| %-*s", max( nMaxLength, qsColumnName.length() ),
            szColumnName );
        qsColumnHeader += qsColumn;
	}
	*pqsHorizSep += "+";
	qsColumnHeader += "|";

    txtResults->append( *pqsHorizSep );
    txtResults->append( qsColumnHeader );
    txtResults->append( *pqsHorizSep );

    return qsColumnHeader.length();
}

void classISQL::getResultsBody( SQLHSTMT hStmt, SWORD nColumns )
{
    QString     	qsLine;
    QString     	qsColumn;
	QString			qsColumnName		= "";
    int         	iRC             	= 0;
    SQLUSMALLINT    nCol            	= 0;
	SQLINTEGER		nIndicator;
	char			szColumn[300];
	int				nRow 				= 0;
	unsigned int	nMaxLength			= 10;
	SQLCHAR			szColumnName[101];	

    // PROCESS ALL ROWS
    while( SQL_SUCCESS == (iRC=SQLFetch(hStmt)) || SQL_SUCCESS_WITH_INFO ==
iRC )
    {
		nRow++;
        qsLine = "";
        // PROCESS ALL COLUMNS
        for( nCol = 1; nCol <= nColumns; nCol++ )
        {
			SQLColAttribute( hStmt, nCol, SQL_DESC_LABEL, szColumnName,
sizeof(szColumnName), 0, 0 );
			SQLColAttribute( hStmt, nCol, SQL_DESC_DISPLAY_SIZE, 0, 0, 0, &nMaxLength );
			qsColumnName = (const char *)szColumnName;
            iRC = SQLGetData( hStmt, nCol, SQL_C_CHAR,
                (SQLPOINTER)szColumn, sizeof(szColumn), &nIndicator );
            if ( iRC == SQL_SUCCESS && nIndicator != SQL_NULL_DATA )
                qsColumn.sprintf( "| %-*s", max( nMaxLength, qsColumnName.length() ), szColumn );
            else
				qsColumn.sprintf( "| %-*s", max( nMaxLength, qsColumnName.length() ), "" );

			qsLine += qsColumn;
        }
		qsLine += "|";
        txtResults->append( qsLine );
    }
}

void classISQL::ChangeTextType( int nTab )
{
    if ( nTab == 0 )
    {
        txtSQL->show();
        txtResults->hide();
        lblFileName->setText( qsSQLFileName );
    }
    else
    {
        txtSQL->hide();
        txtResults->show();
        lblFileName->setText( qsResultsFileName );
    }
}

void classISQL::NewSQL()
{
    txtSQL->clear();
    setCaption( qsDataSource );
    qsSQLFileName = "";
    qsResultsFileName = "";

    setTextType( 0 );
}

void classISQL::OpenSQL()
{
	QMultiLineEdit *txt;

	if ( pTabBar->currentTab() == 0 )
		txt = txtSQL;
	else
		txt = txtResults;

    // LET USER PICK A FILE
    QString qsFile = QFileDialog::getOpenFileName();
    if ( qsFile.isNull() )
	    return;

    // TRY TO LOAD THE FILE
    QFile hFile( qsFile );

    if ( !hFile.open( IO_ReadOnly ) )
	    return;

    txt->setAutoUpdate( FALSE );
    txt->clear();

    QTextStream t( &hFile );
    while ( !t.eof() )
    {
	    QString s = t.readLine();
	    txt->append( s );
    }
    hFile.close();

    txt->setAutoUpdate( TRUE );
    txt->repaint();

	if ( pTabBar->currentTab() == 0 )
		qsSQLFileName = qsFile;
	else
		qsResultsFileName = qsFile;

    setTextType( 0 );
}

void classISQL::SaveSQL()
{
	QMultiLineEdit 	*txt;
	QString			qsFileName;

	if ( pTabBar->currentTab() == 0 )
	{
		txt = txtSQL;
		qsFileName = qsSQLFileName;
	}
	else
	{
		txt = txtResults;
		qsFileName = qsResultsFileName;
	}

    if ( qsFileName == "" )
    {
        SaveAsSQL();
        return;
    }

    // TRY TO SAVE THE FILE
    QFile hFile( qsFileName );

    if ( !hFile.open( IO_WriteOnly ) )
	    return;

    hFile.writeBlock( txt->text(), txt->text().length() );
    hFile.close();

}

void classISQL::SaveAsSQL()
{
	QMultiLineEdit 	*txt;
	QString			qsFileName;

	if ( pTabBar->currentTab() == 0 )
	{
		txt = txtSQL;
		qsFileName = qsSQLFileName;
	}
	else
	{
		txt = txtResults;
		qsFileName = qsResultsFileName;
	}

    // LET USER PICK A FILE
    QString qsFile = QFileDialog::getSaveFileName( qsFileName );
    if ( qsFile.isNull() )
	    return;

    // TRY TO SAVE THE FILE
    QFile hFile( qsFile );

    if ( !hFile.open( IO_WriteOnly ) )
	    return;

    hFile.writeBlock( txt->text(), txt->text().length() );
    hFile.close();

	if ( pTabBar->currentTab() == 0 )
	{
		qsSQLFileName = qsFile;
        lblFileName->setText( qsSQLFileName );
	}
	else
	{
		qsResultsFileName = qsFile;
        lblFileName->setText( qsResultsFileName );
	}

}

void classISQL::setTextType( int nIndex )
{
    pTabBar->setCurrentTab( nIndex );
    if ( nIndex == 0 )
    {
        txtResults->hide();
        txtSQL->show();
        lblFileName->setText( qsSQLFileName );
    }
    else
    {
        txtSQL->hide();
        txtResults->show();
        lblFileName->setText( qsResultsFileName );
    }
}

void classISQL::resizeEvent( QResizeEvent *p )
{
    int nMarg = 1;
    int nYOffset = mbMain->height() + pTabBar->height();
    int nW = p->size().width() - nMarg * 2;
    int nH = p->size().height() - nMarg * 2;

	mbMain->setGeometry( 0, 0, p->size().width(), 30 );
	txtSQL->setGeometry( nMarg, nYOffset + nMarg, nW, nH - nYOffset -
        lblFileName->height() );
	txtResults->setGeometry( nMarg, nYOffset + nMarg, nW, nH - nYOffset -
        lblFileName->height()  );
	lblFileName->setGeometry( nMarg, p->size().height() - nMarg -
        lblFileName->height(), nW, lblFileName->height()  );
}
