/**************************************************
 *
 *
 **************************************************
 * This code was created by Peter Harvey @ CodeByDesign.
 * Released under GPL 18.FEB.99
 *
 * Contributions from...
 * -----------------------------------------------
 * Peter Harvey		- pharvey@codebydesign.com
 **************************************************/
#include "classTable.h"
#include "table.xpm"

classTable::classTable( QListView *pParent, classCanvas *pCanvas, SQLHDBC hDbc, char *pszName )
    : classNode( pParent, pCanvas )
{
	Init( hDbc, pszName );
}

classTable::classTable( QListViewItem *pParent, classCanvas *pCanvas, SQLHENV hDbc, char *pszName )
    : classNode( pParent, pCanvas )
{
	Init( hDbc, pszName );
}

classTable::~classTable()
{
	listColumns.clear();
}

void classTable::Init( SQLHDBC hDbc, char *pszName )
{
	this->hDbc		= hDbc;
	this->qsName	= pszName;
	setText( 0, qsName );
	setText( 1, "" );
	sortChildItems( 1, FALSE );
	listColumns.setAutoDelete( TRUE );
    pPrimaryKeys	= 0;
	pIndexs 		= 0;
	pSpecialColumns	= 0;
	this->setPixmap( 0, QPixmap( table_xpm ) );
}

void classTable::setOpen( bool o )
{
    if ( o && !childCount() )
	{
		LoadColumns();
		pPrimaryKeys	= new classPrimaryKeys( this, pCanvas, hDbc, (char *)qsName.data() );
		pIndexs 		= new classIndexs( this, pCanvas, hDbc, (char *)qsName.data() );
		pSpecialColumns = new classSpecialColumns( this, pCanvas, hDbc, (char *)qsName.data() );
    }
    QListViewItem::setOpen( o );
}

void classTable::setup()
{
    setExpandable( TRUE );
    QListViewItem::setup();
}

void classTable::LoadColumns()
{
	SQLHSTMT        hstmt;
	SQLRETURN		nReturn             = -1;
	SQLCHAR         szTableName[101]	= "";
	SQLCHAR         szColumnName[101]	= "";
	QString         qsError;

	// CREATE A STATEMENT
	nReturn = SQLAllocStmt( hDbc, &hstmt );
	if ( nReturn != SQL_SUCCESS )
	{
		QMessageBox::warning( pCanvas, "Data Manager",  "Failed to SQLAllocStmt" );
		return;
	}

	// EXECUTE OUR SQL/CALL
	strcpy( (char *)szTableName, qsName.data() );

	if ( SQL_SUCCESS != (nReturn=SQLColumns( hstmt, 0, 0, 0, 0, szTableName, SQL_NTS, 0, 0 )) )
	{
		QMessageBox::warning( pCanvas, "Data Manager",  "Failed to SQLColumns" );
		return;
	}

	// GET RESULTS
	nReturn = SQLFetch( hstmt );
	while ( nReturn == SQL_SUCCESS || nReturn == SQL_SUCCESS_WITH_INFO )
	{
		nReturn = SQLGetData( hstmt, 4, SQL_C_CHAR, &szColumnName[0], sizeof(szColumnName), 0 );
		if ( nReturn != SQL_SUCCESS )
			strcpy( (char *)szColumnName, "Unknown" );

		listColumns.append( new classColumn( this, pCanvas, hDbc, (char *)szColumnName ) );

		nReturn = SQLFetch( hstmt );
	}

	// FREE STATEMENT
	nReturn = SQLFreeStmt( hstmt, SQL_DROP );
	if ( nReturn != SQL_SUCCESS )
		QMessageBox::warning( pCanvas, "Data Manager",  "Failed to SQLFreeStmt" );

}

void classTable::selectionChanged( QListViewItem *p )
{
	classColumn	*pColumn;

	for ( pColumn = listColumns.first(); pColumn != 0; pColumn = listColumns.next() )
        pColumn->selectionChanged( p );

	if ( p == this )
	{
	}
}

