/*****************************************************************************
 * SQLFetchScroll
 *
 **********************************************************************
 *
 * This code was created by Peter Harvey (mostly during Christmas 98/99).
 * This code is LGPL. Please ensure that this message remains in future
 * distributions and uses of this code (thats about all I get out of it).
 * - Peter Harvey pharvey@codebydesign.com
 *
 *****************************************************************************/

#include "driver.h"

SQLRETURN SQLFetchScroll(	SQLHSTMT    hDrvStmt,
							SQLSMALLINT nOrientation,
							SQLINTEGER  nOffset )
{
    HDRVSTMT 		hStmt		= (HDRVSTMT)hDrvStmt;
	int				nCol		= -1;
	HBOUNDCOLUMN	pColumnHeader;			

	/* SANITY CHECKS */
	if ( !hStmt )
		return SQL_INVALID_HANDLE;

	sprintf( hStmt->szSqlMsg, "hStmt = $%08lX", hStmt );
    logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );

	if ( !hStmt->hStmtExtras->hBoundCols )
	{
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR: No result set" );
		return SQL_ERROR;
	}

	/* how much do we want to move */
	switch ( nOrientation )
	{
	case SQL_FETCH_NEXT:
		if ( lstEOL( hStmt->hStmtExtras->hStmt->hTableResult->hRows ) )
			lstFirst( hStmt->hStmtExtras->hStmt->hTableResult->hRows );
		else
			lstNext( hStmt->hStmtExtras->hStmt->hTableResult->hRows );
		break;

	case SQL_FETCH_PRIOR:
        lstPrev( hStmt->hStmtExtras->hStmt->hTableResult->hRows );
		break;

	case SQL_FETCH_FIRST:
        lstFirst( hStmt->hStmtExtras->hStmt->hTableResult->hRows );
		break;

	case SQL_FETCH_LAST:
        lstLast( hStmt->hStmtExtras->hStmt->hTableResult->hRows );
		break;

	case SQL_FETCH_ABSOLUTE:
        lstGoto( hStmt->hStmtExtras->hStmt->hTableResult->hRows, nOffset );
		break;

	case SQL_FETCH_RELATIVE:
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, "SQL_ERROR: Relative search not currently supported" );
		return SQL_ERROR;
		break;

	case SQL_FETCH_BOOKMARK:
		return SQL_ERROR;
		break;

	default:
		sprintf( hStmt->szSqlMsg, "SQL_ERROR: Unknown fetch orientation (%d)", nOrientation );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );
		return SQL_ERROR;
	}

	/************************
	 *
	 ************************/
    if ( lstEOL( hStmt->hStmtExtras->hStmt->hTableResult->hRows ) )
		return SQL_NO_DATA;

	/************************
	 * transfer bound column values to bound storage as required
	 ************************/
	nCol = 1;
	lstFirst( hStmt->hStmtExtras->hBoundCols );
	while ( !lstEOL( hStmt->hStmtExtras->hBoundCols ) )
	{
		pColumnHeader = (HBOUNDCOLUMN)lstGet( hStmt->hStmtExtras->hBoundCols );
		if ( pColumnHeader->pTargetValue != NULL )
		{
			if ( _GetData(	hDrvStmt,
							nCol,
							pColumnHeader->nTargetType,
							pColumnHeader->pTargetValue,
							pColumnHeader->nTargetValueMax,
							pColumnHeader->pnLengthOrIndicator ) != SQL_SUCCESS )
			{
				sprintf( hStmt->szSqlMsg, "SQL_ERROR Failed to get data for column %d", nCol );
				logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING, hStmt->szSqlMsg );
				return SQL_ERROR;
			}
		}
		nCol++;
		lstNext( hStmt->hStmtExtras->hBoundCols );
	}

	logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, LOG_INFO, "SQL_SUCCESS" );
    return SQL_SUCCESS;
}


