#include "sqi.h"

int _DumpParsedSelect( HSQISTMT hStmt );
int _DumpParsedDelete( HSQISTMT hStmt );
int _DumpParsedInsert( HSQISTMT hStmt );
int _DumpParsedUpdate( HSQISTMT hStmt );

int _sqiDump( HSQISTMT hStmt )
{
    if ( !hStmt )
	{
		printf( "[SQI]Nothing to dump.\n" );
		return false;
	}
	
	if ( !hStmt->pParsedSQL )
	{
		printf( "[SQI]Nothing to dump.\n" );
		return false;
	}

	switch ( hStmt->nType )
	{
	case sqpselect:
		return _DumpParsedSelect( hStmt );
		break;

	case sqpdelete:
		return _DumpParsedDelete( hStmt );
		break;

	case sqpinsert:
		return _DumpParsedInsert( hStmt );
		break;

	case sqpupdate:
		return _DumpParsedUpdate( hStmt );
		break;

	default:
		printf( "[SQI]Unknown top level structure type (%d). Unable to dump.\n", hStmt->nType );
	}

	return false;
}


int _DumpParsedSelect( HSQISTMT hStmt )
{
	HSQPSELECT		pSelect;
	HSQPCOLUMN		pColumn;
	HSQPCOMPARISON	pComparison;

	pSelect = (HSQPSELECT)hStmt->pParsedSQL;
	printf( "\nBEGIN DUMP (SELECT)\n" );

	/* COLUMNS */
	printf( "\tCOLUMNS...\n" );
	lstFirst( pSelect->hColumns );
	while ( !lstEOL( pSelect->hColumns ) )
	{
        pColumn = lstGet( pSelect->hColumns );
		printf( "\t\t%s.%s\n", pColumn->pszTable, pColumn->pszColumn );
		lstNext( pSelect->hColumns );
	}
	
	/* TABLES */
	printf( "\tTABLES...\n" );
	printf( "\t\t%s\n", pSelect->pszTable );
	
	/* WHERE */
	printf( "\tWHERE...\n" );
	if ( pSelect->hWhere )
	{
		lstFirst( pSelect->hWhere );
		while ( !lstEOL( pSelect->hWhere ) )
		{
			pComparison = (HSQPCOMPARISON)lstGet( pSelect->hWhere );
			printf( "\t\t%s %s %s\n", pComparison->pszLValue, pComparison->pszOperator, pComparison->pszRValue );

			lstNext( pSelect->hWhere );
		}
	}

	printf( "END DUMP\n\n" );

	return true;
}

int _DumpParsedDelete( HSQISTMT hStmt )
{
	HSQPDELETE		pDelete;
	HSQPCOMPARISON	pComparison;

	pDelete = (HSQPDELETE)hStmt->pParsedSQL;
	printf( "\nBEGIN DUMP (DELETE)\n" );

	/* TABLES */
	printf( "\tTABLES...\n" );
	printf( "\t\t%s\n", pDelete->pszTable );
	
	/* WHERE */
	printf( "\tWHERE...\n" );
	if ( pDelete->hWhere )
	{
		lstFirst( pDelete->hWhere );
		while ( !lstEOL( pDelete->hWhere ) )
		{
			pComparison = (HSQPCOMPARISON)lstGet( pDelete->hWhere );
			printf( "\t\t%s %s %s\n", pComparison->pszLValue, pComparison->pszOperator, pComparison->pszRValue );

			lstNext( pDelete->hWhere );
		}
	}
	if ( pDelete->pszCursor )
	{
		printf( "\t\tCURRENT OF %s\n", pDelete->pszCursor );
	}
	printf( "END DUMP\n\n" );

	return true;
}

int _DumpParsedInsert( HSQISTMT hStmt )
{
	HSQPINSERT	pInsert;


	pInsert = (HSQPINSERT)hStmt->pParsedSQL;
	printf( "\nBEGIN DUMP (INSERT)\n" );

	/* TABLE */
	printf( "\tTABLE...\n" );
	printf( "\t\t%s\n", pInsert->pszTable );
	
	/* COLUMNS */
	printf( "\tCOLUMNS...\n" );
	lstFirst( pInsert->hValues );
	while ( !lstEOL( pInsert->hValues ) )
	{
		printf( "\t\t%s\n", (char*)lstGet( pInsert->hValues ) );
		lstNext( pInsert->hValues );
	}

	printf( "END DUMP\n\n" );

	return true;
}

int _DumpParsedUpdate( HSQISTMT hStmt )
{
	HSQPUPDATE		pUpdate;
	HSQPASSIGNMENT	pAssignment;
	HSQPCOMPARISON	pComparison;

	pUpdate = (HSQPUPDATE)hStmt->pParsedSQL;
	printf( "\nBEGIN DUMP (UPDATE)\n" );

	/* TABLE */
	printf( "\tTABLE...\n" );
	printf( "\t\t%s\n", pUpdate->pszTable );
	
	/* ASSIGNMENTS */
	printf( "\tASSIGNMENTS...\n" );
	lstFirst( pUpdate->hAssignments );
	while ( !lstEOL( pUpdate->hAssignments ) )
	{
        pAssignment = (HSQPASSIGNMENT)lstGet( pUpdate->hAssignments );
		printf( "\t\t%s\t= %s\n", pAssignment->pszColumn, pAssignment->pszValue );
		lstNext( pUpdate->hAssignments );
	}

	/* WHERE */
	printf( "\tWHERE...\n" );
	if ( pUpdate->hWhere )
	{
		lstFirst( pUpdate->hWhere );
		while ( !lstEOL( pUpdate->hWhere ) )
		{
			pComparison = (HSQPCOMPARISON)lstGet( pUpdate->hWhere );
			printf( "\t\t%s %s %s\n", pComparison->pszLValue, pComparison->pszOperator, pComparison->pszRValue );

			lstNext( pUpdate->hWhere );
		}
	}
	if ( pUpdate->pszCursor )
	{
		printf( "\t\tCURRENT OF %s\n", pUpdate->pszCursor );
	}

	printf( "END DUMP\n\n" );

	return true;
}


