#include "sqi.h"

int		_sqiExecuteParsedSelect( HSQISTMT hStmt );
int		_sqiExecuteParsedDelete( HSQISTMT hStmt );
int		_sqiExecuteParsedInsert( HSQISTMT hStmt );
int		_sqiExecuteParsedUpdate( HSQISTMT hStmt );
int		_sqiFilterColHdrs( HLST hNewColHdrs, void *pDataRoot );
int		_sqiFilterColHdrsUpdate( HLST hNewColHdrs, void *pDataRoot );
int		_sqiFilterRows( HLST hNewRows, void *pDataRoot );
int		_sqiGetColumnNumber( HLST hColHdrs, char *pszColumn );

HSQICOLUMN	_sqiCopyColumn( HSQICOLUMN pColumn, int nCol );
char **		_sqiCopyRow( char **pRow, HLST hColsSource );

/************************
 * _sqiExecuteParsedSQL
 *
 ************************/
int _sqiExecuteParsedSQL( HSQISTMT hStmt )
{

	if ( !hStmt )
		return false;

	if ( hStmt->hTableResult )
		return false;

	hStmt->nRowsAffected = 0;

	switch ( hStmt->nType )
	{
	case sqpselect:
		return _sqiExecuteParsedSelect( hStmt );
		break;

	case sqpdelete:
		return _sqiExecuteParsedDelete( hStmt );
		break;

	case sqpinsert:
		return _sqiExecuteParsedInsert( hStmt );
		break;

	case sqpupdate:
		return _sqiExecuteParsedUpdate( hStmt );
		break;

	default:
		sprintf( hStmt->szMsg, "Unknown SQL command type (%d)", hStmt->nType );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
	}

	return false;
}


/************************
 * 	We can assume...
 *
 * 1. one table per select
 * 2. no sub selects
 * 3. where clause is a simple "LValue Operator RValue" where LValue is always a column name and RValue is always a string
 * 4. select only consists of a column list, a table, and a simple where clause
 *
 * Lets use liblst's ability to create lists which ref a subset of some other list. We
 * will use for the hColHdrs and hRows.
 ************************/
int _sqiExecuteParsedSelect( HSQISTMT hStmt )
{
	HSQI			hSQI;
	HSQPSELECT		pSelect;	
	HSQPCOMPARISON	pComparison;
	char			szCursorName[51];

	HSQITABLE		hTableRoot;				/* source table 							*/
	HLST			hColsRoot;				/* source cols in source table (cursor)		*/
	HLST			hRowsRoot;				/* source rows in source table (cursor)		*/
	HSQICOLUMN		pColumnRoot;			/* source column header in source cols		*/
	HSQICOLUMN		pColumnResult;			/* dest column header (a copy of source)	*/
	char			**pRowRoot;				/* source row in source rows				*/
	char			**pRowResult;			/* dest row (a copy of source row)			*/

	if ( !hStmt )
		return false;

	if ( !hStmt->pParsedSQL )
		return false;

	hSQI	= (HSQI)hStmt->hSQI;
    pSelect	= (HSQPSELECT)hStmt->pParsedSQL;

	/************************
	 * CLEANUP PREVIOUS RESULTS
	 ************************/
	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	/************************
	 * GET REF TO ROOT TABLE FROM TABLE CACHE
	 ************************/
	hTableRoot = _sqiGetTableHandle( hSQI, pSelect->pszTable );
	if ( !hTableRoot )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pSelect->pszTable );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	/************************
	 * PRE CALC COLUMN INDEX FOR COLUMNS INVOLVED IN WHERE CLAUSE (for reasonable performance)
	 ************************/
	if ( pSelect->hWhere )
	{
		lstFirst( pSelect->hWhere );
		while ( !lstEOL( pSelect->hWhere ) )
		{
            pComparison 			= (HSQPCOMPARISON)lstGet( pSelect->hWhere );
			pComparison->nLColumn	= _sqiGetColumnNumber( hTableRoot->hColHdrs, pComparison->pszLValue );
            lstNext( pSelect->hWhere );
		}
	}

	/************************
	 * CREATE A CURSOR ON THE ROOT TABLE
	 ************************/
	hColsRoot = lstOpenCursor( hTableRoot->hColHdrs, _sqiFilterColHdrs, pSelect->hColumns );
	hRowsRoot = lstOpenCursor( hTableRoot->hRows, _sqiFilterRows, pSelect->hWhere );

	/************************
	 * INIT OUR RESULT SET
	 ************************/
	hStmt->hTableResult					= malloc( sizeof(SQITABLE) );
	sprintf( szCursorName, "TEMP_%p", hStmt->hTableResult );

	hStmt->hTableResult->hBase			= NULL;
	hStmt->hTableResult->hColHdrs		= lstOpen();
    hStmt->hTableResult->hRows			= lstOpen();
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->pszTableName	= (char*)strdup( szCursorName );

	/************************
	 * COPY COLUMN HEADERS
	 ************************/
	lstFirst( hColsRoot );
	while ( !lstEOL( hColsRoot ) )
	{
		pColumnRoot 	= (HSQICOLUMN)lstGet( hColsRoot );
		pColumnResult	= _sqiCopyColumn( pColumnRoot, hStmt->hTableResult->hColHdrs->nItems );
		lstAppend( hStmt->hTableResult->hColHdrs, pColumnResult );
		lstNext( hColsRoot );
	}

	/************************
	 * COPY DATA
	 ************************/
	lstFirst( hRowsRoot );
	while ( !lstEOL( hRowsRoot ) )
	{
		pRowRoot	= (char**)lstGet( hRowsRoot );
		pRowResult	= _sqiCopyRow( pRowRoot, hColsRoot );
		lstAppend( hStmt->hTableResult->hRows, pRowResult );
		lstNext( hRowsRoot );
	}

	/************************
	 * FREE UP OUR CURSORS ON THE ROOT TABLE
	 ************************/
	lstClose( hColsRoot );
	lstClose( hRowsRoot );

	/* we should be EOL for first call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );
	
	hStmt->nRowsAffected = hStmt->hTableResult->hRows->nItems;


	return true;
}


/************************
 * 	We can assume...
 *
 * 1. one table
 * 2. where clause is a simple "LValue Operator RValue" where LValue is always a column name and RValue is always a string
 *
 * Lets use liblst's ability to create lists which ref a subset of some other list. We
 * will use for the hColHdrs and hRows.
 *
 * Then delete them all.
 ************************/
int _sqiExecuteParsedDelete( HSQISTMT hStmt )
{
	HSQI			hSQI;
	HSQITABLE		hTableRoot;
	HSQPDELETE		pDelete	= (HSQPDELETE)hStmt->pParsedSQL;
	HSQPCOMPARISON	pComparison;

	if ( !hStmt )
		return false;

	if ( !hStmt->pParsedSQL )
		return false;

	hSQI	= (HSQI)hStmt->hSQI;

	/************************
	 * A SELECT IS THE ONLY RESULTSET THAT PERSISTS
	 ************************/
	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );


	/************************
	 * SEARCHED DELETE
	 ************************/
	hTableRoot = _sqiGetTableHandle( hSQI, pDelete->pszTable );
	if ( !hTableRoot )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pDelete->pszTable );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}
	hTableRoot->nRefs++;  /* make sure it sticks around until we are done */

	/************************
	 * CREATE A RESULT SET (no col header required)
	 ************************/

	if ( pDelete->hWhere )
	{
		lstFirst( pDelete->hWhere );
		while ( !lstEOL( pDelete->hWhere ) )
		{
            pComparison 			= (HSQPCOMPARISON)lstGet( pDelete->hWhere );
			pComparison->nLColumn	= _sqiGetColumnNumber( hTableRoot->hColHdrs, pComparison->pszLValue );
            lstNext( pDelete->hWhere );
		}
	}


	hStmt->hTableResult					= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->hBase			= hTableRoot;
	hStmt->hTableResult->hColHdrs		= NULL;
    hStmt->hTableResult->hRows			= lstOpenCursor( hTableRoot->hRows, _sqiFilterRows, pDelete->hWhere );
	hStmt->hTableResult->nRefs			= 1;
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->pszTableName	= NULL;

    hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* do the actual delete here */
    lstFirst( hStmt->hTableResult->hRows );
	while ( lstDelete( hStmt->hTableResult->hRows ) )
	{ }

	/************************
	 * CLEANUP
	 ************************/
    sqiFreeResult( hStmt );

	return true;
}


/************************
 * 	We can assume...
 *
 * 1. one table
 * 2. list of strings
 *
 * NOTE: Does insert directly into root table... one of the risky ventures taking place in here <sigh>.
 *
 ************************/
int _sqiExecuteParsedInsert( HSQISTMT hStmt )
{
	HSQI		hSQI;
	HSQITABLE	hTableRoot;
	HSQPINSERT	pInsert	= (HSQPINSERT)hStmt->pParsedSQL;
	char		**aCols;
	int			nCol = 0;

	if ( !hStmt )
		return false;

	if ( !hStmt->pParsedSQL )
		return false;

	hSQI	= (HSQI)hStmt->hSQI;

	/************************
	 * CLEANUP PREVIOUS RESULTS
	 ************************/
	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	/************************
	 * GET REF TO ROOT TABLE FROM TABLE CACHE
	 ************************/
	hTableRoot = _sqiGetTableHandle( hSQI, pInsert->pszTable );
	if ( !hTableRoot )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pInsert->pszTable );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	/************************
	 * BUILD A ROW
	 ************************/
	aCols = malloc( sizeof(char*) * hTableRoot->hColHdrs->nItems + 2 );
	memset( aCols, 0, hTableRoot->hColHdrs->nItems + 2 );

	lstFirst( pInsert->hValues );
	for ( nCol = 0; nCol < hTableRoot->hColHdrs->nItems; nCol++ )
	{
		if ( lstEOL( pInsert->hValues ) )
			aCols[nCol] = (char*)strdup( "" );
		else
			aCols[nCol] = (char*)strdup( lstGet( pInsert->hValues )  );

		lstNext( pInsert->hValues );
	}

	/************************
	 * APPEND ROW
	 ************************/
	lstAppend( hTableRoot->hRows, aCols );

    hStmt->nRowsAffected	= 1;

	return true;
}


/************************
 * 	We can assume...
 *
 * 1. one table
 * 2. where clause is a simple "LValue Operator RValue" where LValue is always a column name and RValue is always a string
 *
 ************************/
int _sqiExecuteParsedUpdate( HSQISTMT hStmt )
{
	HSQI			hSQI;
	HSQITABLE		hTableRoot;
	HSQPUPDATE		pUpdate	= (HSQPUPDATE)hStmt->pParsedSQL;
	HSQPASSIGNMENT	pAssignment;
	HSQICOLUMN		pColumnResult;
	HSQPCOMPARISON	pComparison;

	if ( !hStmt )
		return false;

	if ( !hStmt->pParsedSQL )
		return false;

	hSQI	= (HSQI)hStmt->hSQI;

	/************************
	 * CLEANUP PREVIOUS RESULTS
	 ************************/
	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	/************************
	 * GET REF TO ROOT TABLE FROM TABLE CACHE
	 ************************/
	hTableRoot = _sqiGetTableHandle( hSQI, pUpdate->pszTable );
	if ( !hTableRoot )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pUpdate->pszTable );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}
	hTableRoot->nRefs++;

	/************************
	 * CREATE A RESULT SET
	 ************************/
	if ( pUpdate->hWhere )
	{
		lstFirst( pUpdate->hWhere );
		while ( !lstEOL( pUpdate->hWhere ) )
		{
            pComparison 			= (HSQPCOMPARISON)lstGet( pUpdate->hWhere );
			pComparison->nLColumn	= _sqiGetColumnNumber( hTableRoot->hColHdrs, pComparison->pszLValue );
            lstNext( pUpdate->hWhere );
		}
	}


	hStmt->hTableResult					= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->hBase			= hTableRoot;
	hStmt->hTableResult->hColHdrs		= lstOpenCursor( hTableRoot->hColHdrs, _sqiFilterColHdrsUpdate, pUpdate->hAssignments );
    hStmt->hTableResult->hRows			= lstOpenCursor( hTableRoot->hRows, _sqiFilterRows, pUpdate->hWhere );
	hStmt->hTableResult->nRefs			= 1;
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->pszTableName	= NULL;

	hStmt->nRowsAffected 				= hStmt->hTableResult->hRows->nItems;

	/************************
	 * DO THE UPDATE ON THE CURSOR
	 ************************/
	lstFirst( hStmt->hTableResult->hRows );
	while ( !lstEOL( hStmt->hTableResult->hRows ) )
	{
		lstFirst( hStmt->hTableResult->hColHdrs );
		lstFirst( pUpdate->hAssignments );
		while ( !lstEOL( pUpdate->hAssignments ) && !lstEOL( hStmt->hTableResult->hColHdrs ) )
		{
			pColumnResult	= (HSQICOLUMN)lstGet( hStmt->hTableResult->hColHdrs );
			pAssignment 	= (HSQPASSIGNMENT)lstGet( pUpdate->hAssignments );
			sqiSetColumn( hStmt->hTableResult, pColumnResult->nCol, pAssignment->pszValue );
			lstNext( hStmt->hTableResult->hColHdrs );
			lstNext( pUpdate->hAssignments );
		}
		lstNext( hStmt->hTableResult->hRows );
	}

	/************************
	 * CLEANUP
	 ************************/
    sqiFreeResult( hStmt );

	return true;
}



/***********************
 * _sqiFilterColHdrs - CALLBACK FUNC
 ***********************/
int _sqiFilterColHdrs( HLST hNewColHdrs, void *pDataRoot )
{
	HLST		hColumns;
	HSQPCOLUMN	pColumn;
	HSQICOLUMN	pRootColumn;

	if ( !pDataRoot )
		return false;
	
	if ( !hNewColHdrs )
		return false;
	
	if ( !hNewColHdrs->pExtras )
		return false;
	
    hColumns 		= (HLST)hNewColHdrs->pExtras;
	pRootColumn		= (HSQICOLUMN)pDataRoot;

	/* return true if found in the parsed columns list from the SQL statement */
	lstFirst( hColumns );
	while ( !lstEOL( hColumns ) )
	{
        pColumn = (HSQPCOLUMN)lstGet( hColumns );
		if ( strcmp( pColumn->pszColumn, "*" ) == 0 )
			return true;
		
		if ( strcmp( pRootColumn->pszSQL_DESC_BASE_COLUMN_NAME, pColumn->pszColumn ) == 0 )
			return true;

		lstNext( hColumns );
	}

	return false;
}

/***********************
 * _sqiFilterColHdrsUpdate - CALLBACK FUNC
 ***********************/
int _sqiFilterColHdrsUpdate( HLST hNewColHdrs, void *pDataRoot )
{
	HLST			hAssignments;
	HSQPASSIGNMENT	pAssignment;
	HSQICOLUMN		pRootColumn;

	if ( !pDataRoot )
		return false;
	
	if ( !hNewColHdrs )
		return false;
	
	if ( !hNewColHdrs->pExtras )
		return false;
	
    hAssignments	= (HLST)hNewColHdrs->pExtras;
	pRootColumn		= (HSQICOLUMN)pDataRoot;

	/* return true if found in the parsed columns list from the SQL statement */
	lstFirst( hAssignments );
	while ( !lstEOL( hAssignments ) )
	{
        pAssignment = (HSQPASSIGNMENT)lstGet( hAssignments );
		if ( strcmp( pRootColumn->pszSQL_DESC_BASE_COLUMN_NAME, pAssignment->pszColumn ) == 0 )
			return true;

		lstNext( hAssignments );
	}

	return false;
}


/***********************
 * _sqiFilterRows - CALLBACK FUNC
 *
 * We can assume that all comparisons are linked by the logical AND
 *
 ***********************/
int _sqiFilterRows( HLST hNewRows, void *pDataRoot )
{
	HLST			hWhere;
	HSQPCOMPARISON	pComparison;
	char			**aCols;
	int				nCompare;

	if ( !pDataRoot )
		return false;
	
	if ( !hNewRows )
		return false;
	
	if ( !hNewRows->pExtras ) /* no WHERE so except everything */
		return true;
	
    hWhere 	= (HLST)hNewRows->pExtras;
	aCols	= (char**)pDataRoot;	

	lstFirst( hWhere );
	while ( !lstEOL( hWhere ) )
	{
		pComparison = (HSQPCOMPARISON)lstGet( hWhere );

		if ( pComparison->nLColumn < 0 )
			return false;

		nCompare = strcasecmp( aCols[pComparison->nLColumn], pComparison->pszRValue );

		if ( ( nCompare == 0 && pComparison->pszOperator[0] == '=' ) ||
			 ( nCompare <  0 && pComparison->pszOperator[0] == '<' ) ||
			 ( nCompare >  0 && pComparison->pszOperator[0] == '>' ) ||
			 ( nCompare == 0 && pComparison->pszOperator[1] == '=' ) )
		{
			/* ok */;
		}
		else
            return false;

		lstNext( hWhere );
	}

	return true;
}


/******************************************
 * Knowing the root col number for col names
 * in a where can speed filter up
 ******************************************/
int	_sqiGetColumnNumber( HLST hColHdrs, char *pszColumn )
{
	HSQICOLUMN		pColumn;
	int 			nCol = 0;				/* zero based */
	HLST			hColumns	= lstOpenCursor( hColHdrs, NULL, NULL );

	lstFirst( hColumns );
	while ( !lstEOL( hColumns ) )
	{
		pColumn = (HSQICOLUMN)lstGet( hColumns );
		if ( strcmp( pColumn->pszSQL_DESC_BASE_COLUMN_NAME, pszColumn ) == 0 )
		{
			lstClose( hColumns );
			return nCol;
		}
		
		nCol++;

		lstNext( hColumns );
	}
	lstClose( hColumns );

	return -1;
}


HSQICOLUMN _sqiCopyColumn( HSQICOLUMN pColumn, int nCol )
{
	HSQICOLUMN p = NULL;
	
	if ( pColumn )
	{
		p = _sqiNewColumnHeader( nCol,
								 pColumn->pszSQL_DESC_CATALOG_NAME,
								 pColumn->pszSQL_DESC_TABLE_NAME,
								 pColumn->pszSQL_DESC_BASE_COLUMN_NAME,
								 pColumn->nSQL_DESC_CONCISE_TYPE,
								 pColumn->nSQL_DESC_LENGTH );
		p->bPimaryKey = pColumn->bPimaryKey;
	}

	return p;
}

char **_sqiCopyRow( char **pRow, HLST hColsSource )
{
	char		**p		= malloc( sizeof(char*) * (hColsSource->nItems + 1) );
	int 		nCol	= 0;
	HSQICOLUMN	pCol;

	lstFirst( hColsSource );
	while ( !lstEOL( hColsSource ) )
	{
		pCol = (HSQICOLUMN)lstGet( hColsSource );
		p[nCol] = (char*)strdup( pRow[pCol->nCol] );
		nCol++;
		lstNext( hColsSource );
	}
	p[nCol] = NULL;

	return p;
}

