#include "sqi.h"

int sqiColumns( HSQISTMT hStmt, char *pszTableName )
{
	HSQI		hSQI;
	HSQITABLE	hTable;
	char		szBuffer[101];
	char		**pData;
	HSQICOLUMN	pColumn;

	if ( !hStmt )
		return false;
	
	if ( !pszTableName )
		return false;

	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	hSQI	= (HSQI)hStmt->hSQI;


	/* GET TABLE HANDLE */
	hTable = _sqiGetTableHandle( hSQI, pszTableName );
	if ( !hTable )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pszTableName );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	/* CREATE A RESULTS TABLE */
	hStmt->hTableResult 				= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->pszTableName	= (char*)strdup( "sqiColumns" );
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->hBase			= NULL;

	/* table header */
	hStmt->hTableResult->hColHdrs	= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hColHdrs, _sqiFreeColumnHeader );

	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 0, "system", "sqiColumns", "TABLE_CAT", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 1, "system", "sqiColumns", "TABLE_SCHEM", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 2, "system", "sqiColumns", "TABLE_NAME", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 3, "system", "sqiColumns", "COLUMN_NAME", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 4, "system", "sqiColumns", "DATA_TYPE", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 5, "system", "sqiColumns", "TYPE_NAME", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 6, "system", "sqiColumns", "COLUMN_SIZE", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 7, "system", "sqiColumns", "BUFFER_LENGTH", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 8, "system", "sqiColumns", "NUM_PREC_RADIX", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 9, "system", "sqiColumns", "DECIMAL_DIGITS", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 10, "system", "sqiColumns", "NULLABLE", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 11, "system", "sqiColumns", "REMARKS", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 12, "system", "sqiColumns", "COLUMN_DEF", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 13, "system", "sqiColumns", "SQL_DATA_TYPE", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 14, "system", "sqiColumns", "SQL_DATETIME_SUB", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 15, "system", "sqiColumns", "CHAR_OCTET_LENGTH", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 16, "system", "sqiColumns", "ORDINAL_POSITION", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 17, "system", "sqiColumns", "IS_NULLABLE", SQL_VARCHAR, 50 ) );

	/* table rows (based upon root table col hdrs) */
	hStmt->hTableResult->hRows		= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hRows, _sqiFreeDataRow );

	lstFirst( hTable->hColHdrs );
	while ( !lstEOL( hTable->hColHdrs ) )
	{
		pData		= malloc( sizeof(char*) * 19 );
		pColumn 	= (HSQICOLUMN)lstGet( hTable->hColHdrs );
		pData[0]	= (char *)strdup( pColumn->pszSQL_DESC_CATALOG_NAME );
		pData[1]	= (char *)strdup( pColumn->pszSQL_DESC_SCHEMA_NAME );
		pData[2]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_TABLE_NAME );
		pData[3]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_COLUMN_NAME );
		pData[4]	= (char *)strdup( "" );
		pData[5]	= (char *)strdup( pColumn->pszSQL_DESC_TYPE_NAME );
		sprintf( szBuffer, "%d", pColumn->nSQL_DESC_LENGTH );
		pData[6]	= (char *)strdup( szBuffer );
		pData[7]	= (char *)strdup( "" );
		pData[8]	= (char *)strdup( "" );
		pData[9]	= (char *)strdup( "" );
		pData[10]	= (char *)strdup( (pColumn->nSQL_DESC_NULLABLE ? "0" : "1") );
		pData[11]	= (char *)strdup( "" );
		pData[12]	= (char *)strdup( "" );
		pData[13]	= (char *)strdup( "" );
		pData[14]	= (char *)strdup( "" );
		pData[15]	= (char *)strdup( "" );
		pData[16]	= (char *)strdup( "" );
		pData[17]	= (char *)strdup( "" );

		pData[18]	= NULL; /* data terminator... see _sqiFreeDataRow */

		lstAppend( hStmt->hTableResult->hRows, pData );

		lstNext( hTable->hColHdrs );
	}
	lstClose( hTable->hColHdrs );

	hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* we should be EOL for call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );

	return true;
}


