%{
#include <string.h>
#include <math.h>


#include "sqp.h"

#define YYDEBUG 1

%}
	
	/* symbolic tokens */

%union {
	int intval;
	double floatval;
	char *strval;
	int subtok;
}

	/* operators */


	/* literal keyword tokens */
%token <strval> COMPARISON
%token <strval> NAME
%token <strval> STRING
%token INTNUM APPROXNUM

%token SELECT FROM WHERE
%token DELETE
%token INSERT INTO VALUES
%token UPDATE SET
%token CURRENT OF
%token AND

%%

y_sql:		
		y_select
	|	y_delete
	|	y_insert
	|	y_update
	;
	
y_select:
		SELECT y_columns FROM y_table y_condition	{ sqpStoreSelect(); }
	;

y_delete:
		DELETE FROM y_table y_condition			{ sqpStoreDelete(); }
	|	DELETE y_table y_condition			{ sqpStoreDelete(); }
	;

y_insert:
		INSERT INTO y_table y_values			{ sqpStoreInsert(); }
	|	INSERT y_table y_values				{ sqpStoreInsert(); }
	;

y_update:
		UPDATE y_table SET y_assignments y_condition	{ sqpStoreUpdate(); }
	;

y_columns:
		y_column_list
	|	'*'						{ sqpStoreColumn( "*" ); }
	;
	
y_column_list:
		NAME						{ sqpStoreColumn( $1 ); }
	|	y_column_list ',' NAME				{ sqpStoreColumn( $3 ); }
	;

y_table:
		NAME 						{ sqpStoreTable( $1 ); }
	;
	

y_condition:
		/* optional */
	|	WHERE y_comparisons
	|	WHERE CURRENT OF NAME				{ sqpStorePositioned( $4 ); }
	;
	
y_values:
		VALUES '(' y_value_list ')'
	;

y_value_list:
		STRING						{ sqpStoreValue( $1 ); }
	|	y_value_list ',' STRING				{ sqpStoreValue( $3 ); }
	;

y_assignment:
		NAME COMPARISON STRING				{ sqpStoreAssignment( $1, $3 ); }
	;

y_assignments:
		y_assignment
	|	y_assignments ',' y_assignment
	;

y_comparison:
		NAME COMPARISON STRING				{ sqpStoreComparison( $1, $2, $3 ); }
	;

y_comparisons:
		y_comparison
	|	y_comparisons AND y_comparison
	;

%%


